<?php
/**
 @file
@author Julien George
@brief Gére toutes les exceptions de l'API
@details La classe EsiaException qui hérite de la classe php Exception est la mère de toutes les autres Exceptions. Celles-ci sont classés, via les relations d'héritages, par sources. Exemple : EsiaException -> SocketException -> SocketCreateException.
*/

/**
 @brief Classe mére de toutes les exceptions de l'API
@details Classe mére de toutes les autres classes Exceptions de ce fichier.
*/
abstract class EsiaException extends Exception {
	/** @brief Source de l'exception (TCP,SSL, Remote Device, ...) */
	private $source;

	/**
		@brief Initialise la source et appel le constructeur d'Exception
		@param string $source Le nom de la source de l'erreur (TCP,SSL, Remote Device, ...)
		@param string $message Le message d'erreur. Les "\n" seront remplacé par des " | "
		@param int $code Le code du message d'erreur.
		@param Exception $previous (default = null) Exception qui à causé celle-ci.
		*/
	public function __construct($source,$message, $code, Exception $previous = null) {
		$this->source=$source;
		parent::__construct(str_replace("\n"," | ",$message), (int) $code, $previous);
	}

	/**
		@brief Permet de récupérer la source de l'Erreur
		@return (string) TCP,SSL, Remote Device, ...
		*/
	public function getSource(){
		return $this->source;
	}
}

/**
 @brief Mère de toutes les exceptions de type Socket
 @details Toutes les classes héritant de celle-ci auront comme source d'erreur Socket
 */
abstract class SocketException extends EsiaException{
	/**
	 @brief Appelle le constructeur d'EsiaException
		@details Initialise la source à Socket et prefixe le code avec 10
		@param string $message Le message d'erreur
		@param int $code Le code du message d'erreur.
		@param Exception $previous (default = null) Exception qui à causé celle-ci.
		*/
	public function __construct($message, $code, Exception $previous = null) {
		parent::__construct("Socket",$message,"10$code", $previous);
	}
}

/**
 @brief Généré quand on arrive pas à binder une socket
 */
class SocketBindException extends SocketException{
	/**
	 @brief Appelle le constructeur de SocketException en initialisant le code à 00
	 @param string $message Le message d'erreur
	 @param Exception $previous (default = null) Exception qui à causé celle-ci.
	 */
	public function __construct($message, Exception $previous = null) {
		parent::__construct($message,"00",$previous);
	}
}

/**
 @brief Généré si une socket n'arrive pas à accepter une connection
 */
class SocketAcceptException extends SocketException{
	/**
	 @brief Appelle le constructeur de SocketException en initialisant le code 01
	 @param string $message Le message d'erreur
	 @param Exception $previous (default = null) Exception qui à causé celle-ci.
	 */
	public function __construct($message, Exception $previous = null) {
		parent::__construct($message,01, $previous);
	}
}

/**
 @brief Généré si une socket n'arrive pas à se connecter
 */
class SocketConnectException extends SocketException{
	/**
	 @brief Appelle le constructeur de SocketException en initialisant le code 02
	 @param string $message Le message d'erreur
	 @param Exception $previous (default = null) Exception qui à causé celle-ci.
	 */
	public function __construct($message, Exception $previous = null) {
		parent::__construct($message,02, $previous);
	}
}

/**
 @brief Généré si on essaie d'agir sur une socket non connectée
 */
class SocketClosedException extends SocketException{
	/**
	 @brief Appelle le constructeur de SocketException en initialisant le code 03
	 @param string $message Le message d'erreur
	 @param Exception $previous (default = null) Exception qui à causé celle-ci.
	 */
	public function __construct($message, Exception $previous = null) {
		parent::__construct($message,03, $previous);
	}
}

/**
 @brief Généré si on essaie de connecter une socket qui est déjà connectée
 */
class SocketConnectedException extends SocketException{
	/**
	 @brief Appelle le constructeur de SocketException en initialisant le code 04
	 @param string $message Le message d'erreur
	 @param Exception $previous (default = null) Exception qui à causé celle-ci.
	 */
	public function __construct($message, Exception $previous = null) {
		parent::__construct($message,04, $previous);
	}
}

/**
 @brief Erreur de lecture des données
 */
class SocketReadException extends SocketException{
	/**
	 @brief Appelle le constructeur de SocketException en initialisant le code 05
	 @param string $message Le message d'erreur
	 @param Exception $previous (default = null) Exception qui à causé celle-ci.
	 */
	public function __construct($message, Exception $previous = null) {
		parent::__construct($message,05, $previous);
	}
}

/**
 @brief Erreur d'envoi des données
 */
class SocketWriteException extends SocketException{
	/**
	 @brief Appelle le constructeur de SocketException en initialisant le code 06
	 @param string $message Le message d'erreur
	 @param Exception $previous (default = null) Exception qui à causé celle-ci.
	 */
	public function __construct($message, Exception $previous = null) {
		parent::__construct($message,06, $previous);
	}
}



/**
 @brief Mère de toutes les exceptions de type SSL
 @details Toutes les classes héritant de celle-ci auront comme source d'erreur SSL
 */
abstract class SSLException extends EsiaException{
	/**
	 @brief Appelle le constructeur d'EsiaException
	 @details Initialise la source à SSL et prefixe le code avec 11
	 @param string $message Le message d'erreur
	 @param int $code Le code du message d'erreur.
	 @param Exception $previous (default = null) Exception qui à causé celle-ci.
	 */
	public function __construct($message, $code, Exception $previous = null) {
		parent::__construct("SSL",$message,"11$code", $previous);
	}
}

/**
 @brief Mère de toutes les exceptions de type HTTP
 @details Toutes les classes héritant de celle-ci auront comme source d'erreur HTTP
 */
abstract class HTTPHeaderException extends EsiaException{
	/**
	 @brief Appelle le constructeur d'EsiaException
	 @details Initialise la source à HTTP et prefixe le code avec 12
	 @param string $message Le message d'erreur
	 @param int $code Le code du message d'erreur.
	 @param Exception $previous (default = null) Exception qui à causé celle-ci.
	 */
	public function __construct($message, $code, Exception $previous = null) {
		parent::__construct("HTTPHeader",$message,"12$code", $previous);
	}
}

/**
 @brief Généré si le header HTTP n'est pas un header websocket correct
 */
class BadHeaderException extends HTTPHeaderException{
	/**
	 @brief Appelle le constructeur de HTTPHeaderException en initialisant le code 00
	 @param string $message Le message d'erreur
	 @param Exception $previous (default = null) Exception qui à causé celle-ci.
	 */
	public function __construct($message, Exception $previous = null) {
		parent::__construct($message,00, $previous);
	}
}

/**
 @brief Généré en cas d'erreur dans le Header HTTP
 */
class BadVersionException extends HTTPHeaderException{
	/**
	 @brief Appelle le constructeur de HTTPHeaderException en initialisant le code 01
	 @param string $message Le message d'erreur
	 @param Exception $previous (default = null) Exception qui à causé celle-ci.
	 */
	public function __construct($message, Exception $previous = null) {
		parent::__construct($message,01, $previous);
	}
}

/**
 @brief Généré en cas d'erreur dans la vérification de la clé
 */
class BadKeysException extends HTTPHeaderException{
	/**
	 @brief Appelle le constructeur de HTTPHeaderException en initialisant le code 02
	 @param string $message Le message d'erreur
	 @param Exception $previous (default = null) Exception qui à causé celle-ci.
	 */
	public function __construct($message, Exception $previous = null) {
		parent::__construct($message,02, $previous);
	}
}

/**
 @brief Généré si les sous-protocoles du client ne sont pas connus par le serveur
 */

class BadProtocolsException extends HTTPHeaderException{
	/**
	 @brief Appelle le constructeur de HTTPHeaderException en initialisant le code 03
	 @param string $message Le message d'erreur
	 @param Exception $previous (default = null) Exception qui à causé celle-ci.
	 */
	public function __construct($message, Exception $previous = null) {
		parent::__construct($message,03, $previous);
	}
}


/**
 @brief Généré Lorsque le serveur envoi une réponse HTTP à laquelle on ne s'attends pas.
 */
class BadHttpResponseException extends HTTPHeaderException{
	/** @brief Code HTTP reçu. */
	private $received_code;
	/** @brief Signification du HTTP reçu. */
	private $received_type;
	/** @brief Code HTTP quon aurait souhaité recevoir. */
	private $expected_code;
	/** @brief Signification du code HTTP souhaité. */
	private $expected_type;

	/**
	 @brief Appelle le constructeur de HTTPHeaderException en initialisant le code 04.
		@details Le message d'erreur est généré automatiquement à partir des paramètres passés.
		@param int $received_code Code HTTP de la réponse reçue.
		@param string $received_type Signification du code HTTP de la réponse.
		@param int $expected_code Code HTTP qu'on souhaitait recevoir.
		@param string $expected_type Signification du code HTTP $expected_code.
		*/
	public function __construct($received_code,$received_type,$expected_code,$expected_type){
		$this->received_code=$received_code;
		$this->received_type=$received_type;
		$this->expected_code=$expected_code;
		$this->expected_type=$expected_type;

		$message="Received code $received_code : $received_type. Expected $received_code : $received_type.";
		parent::__construct($message,04);
	}

	/**
	 @brief Retourne le code HTTP reçu.
	 @return (int) 404, 400, ...
	 */
	public function getReceived_code(){
		return $this->received_code;
	}

	/**
	 @brief Retourne la signification du code HTTP reçu.
	 @return (string) Not Found, Bad Request, ...
	 */
	public function getReceived_type(){
		return $this->received_type;
	}

	/**
	 @brief Retourne le code HTTP qu'on souhaitait recevoir.
	 @return (int) 101, 200, ...
	 */
	public function getExpected_code(){
		return $this->expected_code;
	}

	/**
	 @brief Retourne la signification du code HTTP $expected_code.
	 @return (int) Switching Protocols, OK, ...
	 */
	public function getExpected_type(){
		$this->expected_type;
	}
}

/**
 @brief Mère de toutes les exceptions de type WS
 @details Toutes les classes héritant de celle-ci auront comme source d'erreur WS
 */
abstract class WSException extends EsiaException{
	/**
	 @brief Appelle le constructeur d'EsiaException
	 @details Initialise la source à WS et prefixe le code avec 13
	 @param string $message Le message d'erreur
	 @param int $code Le code du message d'erreur.
	 @param Exception $previous (default = null) Exception qui à causé celle-ci.
	 */
	public function __construct($message, $code, Exception $previous = null) {
		parent::__construct("WS",$message,"13$code", $previous);
	}
}
/**
 @brief Erreur de base de donnée
 */
class EsiaDBException extends EsiaException{
	/**
	 @brief Appelle le constructeur de SocketException en initialisant le code 06
	 @param string $message Le message d'erreur
	 @param Exception $previous (default = null) Exception qui à causé celle-ci.
	 */
	public function __construct($message, Exception $previous = null) {
		parent::__construct($message,1, $previous);
	}
}
?>